@@ -45,7 +45,7 @@ import antlr.collections.ASTEnumeration;
  *
  */
 public class StringTemplate {
-	public static final String VERSION = "3.1"; // January 23, 2008
+	public static final String VERSION = "3.2.1"; // September 22, 2009
 
 	/** <@r()> */
 	public static final int REGION_IMPLICIT = 1;
@@ -66,7 +66,7 @@ public class StringTemplate {
 	 *
 	 *  using template:
 	 *
-	 *  $items:{$attr.name$=$attr.type$}$
+	 *  $items:{it.name$=$it.type$}$
 	 *
 	 *  This example will call getName() on the objects in items attribute, but
 	 *  what if they aren't objects?  I have perhaps two parallel arrays
@@ -147,9 +147,6 @@ public class StringTemplate {
 	 */
 	protected StringTemplate enclosingInstance = null;
 
-	/** A list of embedded templates */
-	protected List embeddedInstances = null;
-
 	/** If this template is an embedded template such as when you apply
 	 *  a template to an attribute, then the arguments passed to this
 	 *  template represent the argument context--a set of values
@@ -378,17 +375,6 @@ public class StringTemplate {
 		}
 		// set the parent for this template
 		this.enclosingInstance = enclosingInstance;
-		// make the parent track this template as an embedded template
-		if ( enclosingInstance!=null ) {
-			this.enclosingInstance.addEmbeddedInstance(this);
-		}
-	}
-
-	public void addEmbeddedInstance(StringTemplate embeddedInstance) {
-		if ( this.embeddedInstances==null ) {
-			this.embeddedInstances = new LinkedList();
-		}
-		this.embeddedInstances.add(embeddedInstance);
 	}
 
 	public Map getArgumentContext() {
@@ -481,7 +467,7 @@ public class StringTemplate {
 	}
 
 	public void removeAttribute(String name) {
-		attributes.remove(name);
+		if ( attributes!=null ) attributes.remove(name);
 	}
 
 	/** Set an attribute for this template.  If you set the same
@@ -693,7 +679,11 @@ public class StringTemplate {
 	}
 
 	public Object getAttribute(String name) {
-		return get(this,name);
+		Object v = get(this,name);
+        if ( v==null ) {
+            checkNullAttributeAgainstFormalArguments(this, name);
+        }
+        return v;
 	}
 
 	/** Walk the chunks, asking them to write themselves out according
@@ -707,13 +697,14 @@ public class StringTemplate {
 			group.emitTemplateStartDebugString(this,out);
 		}
 		int n = 0;
+        boolean missing = true;
 		setPredefinedAttributes();
 		setDefaultArgumentValues();
 		for (int i=0; chunks!=null && i<chunks.size(); i++) {
 			Expr a = (Expr)chunks.get(i);
 			int chunkN = a.write(this, out);
 			// expr-on-first-line-with-no-output NEWLINE => NEWLINE
-			if ( chunkN==0 && i==0 && (i+1)<chunks.size() &&
+			if ( chunkN<=0 && i==0 && (i+1)<chunks.size() &&
 				 chunks.get(i+1) instanceof NewlineRef )
 			{
 				//System.out.println("found pure first-line-blank \\n pattern");
@@ -723,22 +714,24 @@ public class StringTemplate {
 			// NEWLINE expr-with-no-output NEWLINE => NEWLINE
 			// Indented $...$ have the indent stored with the ASTExpr
 			// so the indent does not come out as a StringRef
-			if ( chunkN==0 &&
+			if ( chunkN<=0 &&
 				(i-1)>=0 && chunks.get(i-1) instanceof NewlineRef &&
 				(i+1)<chunks.size() && chunks.get(i+1) instanceof NewlineRef )
 			{
 				//System.out.println("found pure \\n blank \\n pattern");
 				i++; // make it skip over the next chunk, the NEWLINE
 			}
-			n += chunkN;
+            if ( chunkN!=ASTExpr.MISSING ) {
+                n += chunkN;
+                missing = false;
+            }
 		}
 		if ( group.debugTemplateOutput ) {
 			group.emitTemplateStopDebugString(this,out);
 		}
-		if ( lintMode ) {
-			checkForTrouble();
-		}
-		return n;
+		if ( lintMode ) checkForTrouble();
+		if ( missing && chunks!=null && chunks.size()>0 ) return ASTExpr.MISSING;
+        return n;
 	}
 
 	/** Resolve an attribute reference.  It can be in four possible places:
@@ -768,8 +761,10 @@ public class StringTemplate {
 	 *  This method is not static so people can override functionality.
 	 */
 	public Object get(StringTemplate self, String attribute) {
-		//System.out.println("### get("+self.getEnclosingInstanceStackString()+", "+attribute+")");
-		//System.out.println("attributes="+(self.attributes!=null?self.attributes.keySet().toString():"none"));
+        /*
+		System.out.println("### get("+self.getEnclosingInstanceStackString()+", "+attribute+")");
+		System.out.println("attributes="+(self.attributes!=null?self.attributes.keySet().toString():"none"));
+		*/
 		if ( self==null ) {
 			return null;
 		}
@@ -805,14 +800,16 @@ public class StringTemplate {
 
 		// not locally defined, check enclosingInstance if embedded
 		if ( o==null && self.enclosingInstance!=null ) {
-			/*
+            /*
             System.out.println("looking for "+getName()+"."+attribute+" in super="+
                     enclosingInstance.getName());
-			*/
+             */
 			Object valueFromEnclosing = get(self.enclosingInstance, attribute);
+            /*
 			if ( valueFromEnclosing==null ) {
 				checkNullAttributeAgainstFormalArguments(self, attribute);
 			}
+			*/
 			o = valueFromEnclosing;
 		}
 
@@ -943,6 +940,7 @@ public class StringTemplate {
 	 *  the template attributes table just for consistency's sake.
 	 */
 	public void setDefaultArgumentValues() {
+        //System.out.println("setDefaultArgumentValues; "+name+": argctx="+argumentContext+", n="+numberOfDefaultArgumentValues);
 		if ( numberOfDefaultArgumentValues==0 ) {
 			return;
 		}
@@ -950,6 +948,7 @@ public class StringTemplate {
 			argumentContext = new HashMap();
 		}
 		if ( formalArguments!=FormalArgument.UNKNOWN ) {
+            //System.out.println("formal args="+formalArguments.keySet());
 			Set argNames = formalArguments.keySet();
 			for (Iterator it = argNames.iterator(); it.hasNext();) {
 				String argName = (String) it.next();
@@ -957,13 +956,31 @@ public class StringTemplate {
 				FormalArgument arg =
 					(FormalArgument)formalArguments.get(argName);
 				if ( arg.defaultValueST!=null ) {
+                    //System.out.println("default value="+arg.defaultValueST.chunks);
+                    //System.out.println(getEnclosingInstanceStackString()+": get "+argName+" argctx="+argumentContext);
 					Object existingValue = getAttribute(argName);
+                    //System.out.println("existing value="+existingValue);
 					if ( existingValue==null ) { // value unset?
+                        Object defaultValue = arg.defaultValueST;
 						// if no value for attribute, set arg context
 						// to the default value.  We don't need an instance
 						// here because no attributes can be set in
 						// the arg templates by the user.
-						argumentContext.put(argName, arg.defaultValueST);
+                        int nchunks = arg.defaultValueST.chunks.size();
+                        if ( nchunks==1 ) {
+                            // If default arg is template with single expression
+                            // wrapped in parens, x={<(...)>}, then eval to string
+                            // rather than setting x to the template for later
+                            // eval.
+                            Object a = arg.defaultValueST.chunks.get(0);
+                            if ( a instanceof ASTExpr ) {
+                                ASTExpr e = (ASTExpr)a;
+                                if ( e.getAST().getType()==ActionEvaluator.VALUE ) {
+                                    defaultValue = e.evaluateExpression(this, e.getAST());
+                                }
+                            }
+                        }
+                        argumentContext.put(argName, defaultValue);
 					}
 				}
 			}
@@ -1004,6 +1021,10 @@ public class StringTemplate {
 	}
 
 	public void defineFormalArgument(String name, StringTemplate defaultValue) {
+        /*
+        System.out.println("define formal arg "+this.name+"."+name+
+                           ", def value="+(defaultValue!=null?defaultValue.chunks:"null"));
+                           */
 		if ( defaultValue!=null ) {
 			numberOfDefaultArgumentValues++;
 		}
