@@ -45,6 +45,7 @@ import java.util.*;
  *  parsed into an AST chunk to be evaluated.
  */
 public class ASTExpr extends Expr {
+    public static final int MISSING = -1; // writing -1 char means missing not empty
     public static final String DEFAULT_ATTRIBUTE_NAME = "it";
     public static final String DEFAULT_ATTRIBUTE_NAME_DEPRECATED = "attr";
 	public static final String DEFAULT_INDEX_VARIABLE_NAME = "i";
@@ -133,12 +134,12 @@ public class ASTExpr extends Expr {
         if ( exprTree==null || self==null || out==null ) {
             return 0;
         }
-		out.pushIndentation(getIndentation());
 		// handle options, anchor, wrap, separator...
 		StringTemplateAST anchorAST = (StringTemplateAST)getOption("anchor");
 		if ( anchorAST!=null ) { // any non-empty expr means true; check presence
 			out.pushAnchorPoint();
 		}
+        out.pushIndentation(getIndentation());
 		handleExprOptions(self);
 		//System.out.println("evaluating tree: "+exprTree.toStringList());
         ActionEvaluator eval =
@@ -396,7 +397,7 @@ public class ASTExpr extends Expr {
      */
 	public Object getObjectProperty(StringTemplate self,
 									final Object o,
-									final String propertyName) {
+									final Object propertyName) {
 		if ( o==null || propertyName==null ) {
 			return null;
 		}
@@ -420,14 +421,15 @@ public class ASTExpr extends Expr {
 		return value;
 	}
 
-	protected Object rawGetObjectProperty(StringTemplate self, Object o, String propertyName) {
+	protected Object rawGetObjectProperty(StringTemplate self, Object o, Object property) {
 		Class c = o.getClass();
         Object value = null;
 
-        // Special case: our automatically created Aggregates via
+		// Special case: our automatically created Aggregates via
         // attribute name: "{obj.{prop1,prop2}}"
         if ( c==StringTemplate.Aggregate.class ) {
-            value = ((StringTemplate.Aggregate)o).get(propertyName);
+			String propertyName = (String)property;
+            value = ((StringTemplate.Aggregate)o).get((String)propertyName);
 			return value;
         }
 
@@ -437,6 +439,7 @@ public class ASTExpr extends Expr {
         else if ( c==StringTemplate.class ) {
             Map attributes = ((StringTemplate)o).getAttributes();
             if ( attributes!=null ) {
+				String propertyName = (String)property;
                 value = attributes.get(propertyName);
 				return value;
             }
@@ -446,14 +449,18 @@ public class ASTExpr extends Expr {
         // key not the property method.
         if ( o instanceof Map ) {
             Map map = (Map)o;
-			if ( propertyName.equals("keys") ) {
+			if ( property.equals("keys") ) {
 				value = map.keySet();
 			}
-			else if ( propertyName.equals("values") ) {
+			else if ( property.equals("values") ) {
 				value = map.values();
 			}
-			else if ( map.containsKey(propertyName) ) {
-				value = map.get(propertyName);
+			else if ( map.containsKey(property) ) {
+				value = map.get(property);
+			}
+			else if ( map.containsKey(property.toString()) ) {
+				// if we can't find the key, toString it
+				value = map.get(property.toString());				
 			}
 			else {
 				if ( map.containsKey(DEFAULT_MAP_VALUE_NAME) ) {
@@ -461,7 +468,7 @@ public class ASTExpr extends Expr {
 				}
 			}
 			if ( value == MAP_KEY_VALUE ) {
-				value = propertyName;
+				value = property;
 			}
 			return value;
         }
@@ -494,6 +501,7 @@ public class ASTExpr extends Expr {
 		*/
 
 		// must look up using reflection
+		String propertyName = (String)property;
 		String methodSuffix = Character.toUpperCase(propertyName.charAt(0))+
 			propertyName.substring(1,propertyName.length());
 		m = getMethod(c,"get"+methodSuffix);
@@ -663,103 +671,16 @@ public class ASTExpr extends Expr {
     {
         if ( o==null ) {
 			if ( nullValue==null ) {
-				return 0;
+				return MISSING;
 			}
 			o = nullValue; // continue with null option if specified
-		}
-		int n = 0;
+        }
+        int n = 0;
         try {
-            if ( o instanceof StringTemplate ) {
-                StringTemplate stToWrite = (StringTemplate)o;
-				// failsafe: perhaps enclosing instance not set
-				// Or, it could be set to another context!  This occurs
-				// when you store a template instance as an attribute of more
-				// than one template (like both a header file and C file when
-				// generating C code).  It must execute within the context of
-				// the enclosing template.
-				stToWrite.setEnclosingInstance(self);
-                // if self is found up the enclosing instance chain, then
-                // infinite recursion
-                if ( StringTemplate.inLintMode() &&
-                     StringTemplate.isRecursiveEnclosingInstance(stToWrite) )
-                {
-                    // throw exception since sometimes eval keeps going
-                    // even after I ignore this write of o.
-                    throw new IllegalStateException("infinite recursion to "+
-                            stToWrite.getTemplateDeclaratorString()+" referenced in "+
-                            stToWrite.getEnclosingInstance().getTemplateDeclaratorString()+
-                            "; stack trace:\n"+stToWrite.getEnclosingInstanceStackTrace());
-                }
-                else {
-					// if we have a wrap string, then inform writer it
-					// might need to wrap
-					if ( wrapString!=null ) {
-						n = out.writeWrapSeparator(wrapString);
-					}
-					// check if formatting needs to be applied to the stToWrite
-					if ( formatString != null ) {
-						AttributeRenderer renderer =
-							self.getAttributeRenderer(String.class);
-						if ( renderer != null ) {
-							// you pay a penalty for applying format option to a template
-							// because the template must be written to a temp StringWriter so it can
-							// be formatted before being written to the real output.
-							StringWriter buf = new StringWriter();
-							StringTemplateWriter sw =
-								self.getGroup().getStringTemplateWriter(buf);
-							stToWrite.write(sw);
-							n = out.write(renderer.toString(buf.toString(), formatString));
-							return n;
-						}
-					}
-                    n = stToWrite.write(out);
-                }
-                return n;
-            }
-            // normalize anything iteratable to iterator
-			o = convertAnythingIteratableToIterator(o);
-			if ( o instanceof Iterator ) {
-				Iterator iter = (Iterator)o;
-				boolean seenPrevValue = false;
-				while ( iter.hasNext() ) {
-                    Object iterValue = iter.next();
-					if ( iterValue==null ) {
-						iterValue = nullValue;
-					}
-					if ( iterValue!=null ) {
-						if ( seenPrevValue /*prevIterValue!=null*/
-							&& separatorString!=null ) {
-							n += out.writeSeparator(separatorString);
-						}
-						seenPrevValue = true;
-						int nw = write(self, iterValue, out);
-						n += nw;
-					}
-				}
-			}
-			else {
-				AttributeRenderer renderer =
-					self.getAttributeRenderer(o.getClass());
-				String v = null;
-				if ( renderer!=null ) {
-					if ( formatString != null ) {
-						v = renderer.toString(o, formatString);
-					}
-					else {
-						v = renderer.toString(o);
-					}
-				}
-				else {
-					v = o.toString();
-				}
-				if ( wrapString!=null ) {
-					n = out.write(v, wrapString);
-				}
-				else {
-					n = out.write( v );
-				}
-				return n;
-            }
+            if ( o instanceof StringTemplate ) return writeTemplate(self, o, out);
+            o = convertAnythingIteratableToIterator(o); // normalize
+            if ( o instanceof Iterator ) return writeIterableValue(self, o, out);
+            return writePOJO(self, o, out);
         }
         catch (IOException io) {
             self.error("problem writing object: "+o, io);
@@ -767,15 +688,166 @@ public class ASTExpr extends Expr {
 		return n;
     }
 
-	/** A expr is normally just a string literal, but is still an AST that
+    protected int writePOJO(StringTemplate self, Object o, StringTemplateWriter out) throws IOException {
+        int n = 0;
+        AttributeRenderer renderer =
+            self.getAttributeRenderer(o.getClass());
+        String v = null;
+        if ( renderer!=null ) {
+            if ( formatString != null ) v = renderer.toString(o, formatString);
+            else v = renderer.toString(o);
+        }
+        else v = o.toString();
+        if ( wrapString!=null ) n = out.write(v, wrapString);
+        else n = out.write( v );
+        return n;
+    }
+
+    protected int writeTemplate(StringTemplate self, Object o, StringTemplateWriter out) throws IOException {
+        int n = 0;
+        StringTemplate stToWrite = (StringTemplate)o;
+        // failsafe: perhaps enclosing instance not set
+        // Or, it could be set to another context!  This occurs
+        // when you store a template instance as an attribute of more
+        // than one template (like both a header file and C file when
+        // generating C code).  It must execute within the context of
+        // the enclosing template.
+        stToWrite.setEnclosingInstance(self);
+        // if self is found up the enclosing instance chain, then
+        // infinite recursion
+        if ( StringTemplate.inLintMode() &&
+             StringTemplate.isRecursiveEnclosingInstance(stToWrite) )
+        {
+            // throw exception since sometimes eval keeps going
+            // even after I ignore this write of o.
+            throw new IllegalStateException("infinite recursion to "+
+                    stToWrite.getTemplateDeclaratorString()+" referenced in "+
+                    stToWrite.getEnclosingInstance().getTemplateDeclaratorString()+
+                    "; stack trace:\n"+stToWrite.getEnclosingInstanceStackTrace());
+        }
+        else {
+            // if we have a wrap string, then inform writer it
+            // might need to wrap
+            if ( wrapString!=null ) {
+                n = out.writeWrapSeparator(wrapString);
+            }
+            // check if formatting needs to be applied to the stToWrite
+            if ( formatString != null ) {
+                AttributeRenderer renderer =
+                    self.getAttributeRenderer(String.class);
+                if ( renderer != null ) {
+                    // you pay a penalty for applying format option to a template
+                    // because the template must be written to a temp StringWriter so it can
+                    // be formatted before being written to the real output.
+                    StringWriter buf = new StringWriter();
+                    StringTemplateWriter sw =
+                        self.getGroup().getStringTemplateWriter(buf);
+                    stToWrite.write(sw);
+                    n = out.write(renderer.toString(buf.toString(), formatString));
+                    return n;
+                }
+            }
+            n = stToWrite.write(out);
+        }
+        return n;
+    }
+
+    protected int writeIterableValue(StringTemplate self,
+                                     Object o,
+                                     StringTemplateWriter out)
+        throws IOException
+    {
+        int n = 0;
+        Iterator iter = (Iterator)o;
+        boolean seenAValue = false;
+        while ( iter.hasNext() ) {
+            Object iterValue = iter.next();
+            if ( iterValue==null ) iterValue = nullValue;
+            if ( iterValue!=null ) {
+                // if no separator or separator but iterValue isn't
+                // a single IF condition template
+                if ( separatorString==null ) {
+                    // if no separator, don't waste time writing to
+                    // temp buffer
+                    int nw = write(self, iterValue, out);
+                    if ( nw!=MISSING ) n += nw;
+                    continue;
+                }
+                // if value to emit is a template, only buffer its
+                // value if it's nullable (can eval to missing).
+                // Only a sequence of IF can eval to missing.
+                if ( iterValue instanceof StringTemplate ) {
+                    StringTemplate st = (StringTemplate)iterValue;
+                    int nchunks = st.getChunks()!=null ? st.getChunks().size() : 0;
+                    boolean nullable = true;
+                    for (int i=0; i<nchunks; i++) {
+                        Expr a = (Expr)st.getChunks().get(i);
+                        if ( !(a instanceof ConditionalExpr) ) nullable = false;
+                    }
+                    // if not all IF, not nullable, spit out w/o buffering
+                    if ( !nullable) {
+                        if ( seenAValue && separatorString!=null ) {
+                            n += out.writeSeparator(separatorString);
+                        }
+                        int nw = write(self, iterValue, out);
+                        n += nw;
+                        seenAValue = true;
+                        continue;
+                    }
+                }
+
+                if ( !(iterValue instanceof StringTemplate) &&
+                     !(iterValue instanceof Iterator))
+                {
+                    // if not possible to be missing, don't waste time
+                    // writing to temp buffer; might need separator though
+                    if ( seenAValue && separatorString!=null ) {
+                        n += out.writeSeparator(separatorString);
+                    }
+                    int nw = write(self, iterValue, out);
+                    seenAValue = true;
+                    n += nw;
+                    continue;
+                }
+
+                // if separator exists, write iterated value to a
+                // tmp buffer in case we don't need a separator.
+                // Can't generate separator then test next expr value
+                // as we can't undo separator emit.
+                // Write to dummy buffer to if it is MISSING
+                // but eval/write value again to real out so
+                // we get proper autowrap etc...
+                // Ack: you pay a penalty now for a separator
+                // Later, i can optimze to check if one chunk and
+                // it's a conditional
+                StringWriter buf = new StringWriter();
+                StringTemplateWriter sw =
+                    self.getGroup().getStringTemplateWriter(buf);
+                int tmpsize = write(self, iterValue, sw);
+
+                if ( tmpsize!=MISSING ) {
+                    if ( seenAValue && separatorString!=null ) {
+                        n += out.writeSeparator(separatorString);
+                    }
+                    // do it to real output stream now
+                    int nw = write(self, iterValue, out);
+                    n += nw;
+                    seenAValue = true;
+                }
+            }
+        }
+        return n;
+    }
+
+    /** A expr is normally just a string literal, but is still an AST that
      *  we must evaluate.  The expr can be any expression such as a template
      *  include or string cat expression etc...  Evaluate with its own writer
-	 *  so that we can convert to string and then reuse, don't want to compute
-	 *  all the time; must precompute w/o writing to output buffer.
+     *  so that we can convert to string and then reuse, don't want to compute
+     *  all the time; must precompute w/o writing to output buffer.
      */
-    protected String evaluateExpression(StringTemplate self,
-										Object expr)
-	{
+    public String evaluateExpression(StringTemplate self,
+                                     Object expr)
+    {
         if ( expr ==null ) {
             return null;
         }
@@ -874,7 +946,7 @@ public class ASTExpr extends Expr {
 		return value;
 	}
 
-	private static Object convertAnythingIteratableToIterator(Object o) {
+	protected static Object convertAnythingIteratableToIterator(Object o) {
 		Iterator iter = null;
 		if ( o instanceof Collection ) {
 			iter = ((Collection)o).iterator();
@@ -891,7 +963,7 @@ public class ASTExpr extends Expr {
 		return iter;
 	}
 
-	static Iterator convertAnythingToIterator(Object o) {
+	protected static Iterator convertAnythingToIterator(Object o) {
 		Iterator iter = null;
 		if ( o instanceof Collection ) {
 			iter = ((Collection)o).iterator();
@@ -939,15 +1011,17 @@ public class ASTExpr extends Expr {
 		Object theRest = attribute;
 		attribute = convertAnythingIteratableToIterator(attribute);
 		if ( attribute instanceof Iterator ) {
+			List a = new ArrayList();
 			Iterator it = (Iterator)attribute;
-			if ( !it.hasNext() ) {
-				return null; // if not even one value return null
+            if ( !it.hasNext() ) {
+                return null; // if not even one value return null
+            }
+            it.next(); // ignore first value
+			while (it.hasNext()) {
+				Object o = (Object) it.next();
+				if ( o!=null ) a.add(o);
 			}
-			it.next(); // ignore first value
-			if ( !it.hasNext() ) {
-				return null; // if not more than one value, return null
-			}
-			theRest = it;    // return suitably altered iterator
+			return a;
 		}
 		else {
 			theRest = null;  // rest of single-valued attribute is null
@@ -977,21 +1051,40 @@ public class ASTExpr extends Expr {
 		return last;
 	}
 
-	/** Return an iterator that skips all null values. */
+	/** Return a new list w/o null values. */
 	public Object strip(Object attribute) {
 		if ( attribute==null ) {
 			return null;
 		}
 		attribute = convertAnythingIteratableToIterator(attribute);
 		if ( attribute instanceof Iterator ) {
-			return new StripIterator((Iterator)attribute);
+			List a = new ArrayList();
+			Iterator it = (Iterator)attribute;
+			while (it.hasNext()) {
+				Object o = (Object) it.next();
+				if ( o!=null ) a.add(o);
+			}
+			return a;
 		}
 		return attribute; // strip(x)==x when x single-valued attribute
 	}
 
 	/** Return all but the last element.  trunc(x)=null if x is single-valued. */
 	public Object trunc(Object attribute) {
-		return null; // not impl.
+		if ( attribute==null ) {
+			return null;
+		}
+		attribute = convertAnythingIteratableToIterator(attribute);
+		if ( attribute instanceof Iterator ) {
+			List a = new ArrayList();
+			Iterator it = (Iterator)attribute;
+			while (it.hasNext()) {
+				Object o = (Object) it.next();
+				if ( it.hasNext() ) a.add(o); // only add if not last one
+			}
+			return a;
+		}
+		return null; // trunc(x)==null when x single-valued attribute
 	}
 
 	/** Return the length of a multiple valued attribute or 1 if it is a
